{------------------------------------------------------------------------------
#
#	HourGlass
#		- a sample MIDI Management Tools application
#
#	HourGlass.p	-	Pascal Source
#
#	Copyright  1988-1989 Apple Computer, Inc.
#	All rights reserved.
#
#	Version:	1.0
#
#	HourGlass is a sample application that uses the MIDI Management
#	Tools.  Allows one to display time in all formats.  The time can
#	be internally or externally generated.  If internal, there are
#	start, stop, and zero controls.
#
#	Please be familiar with both the MIDI Management Tools and MIDI
#	in general before trying to understand this code.  
#
#	This application is an example of how to call the MIDI Management
#	tools.  It is not meant as an example of a good Macintosh
#	application; it is NOT a template. It is NOT intended to be 
#	used as a foundation for the next world-class, best-selling, 
#	600K application. A stick figure drawing of the human body may 
#	be a good example of the form for a painting, but that does not 
#	mean it should be used as the basis for the next Mona Lisa.
#
#	This code also makes use of a small library called OneDialog.
#	This is a small set of routines that are only useful for very 
#	simple applications.  Again, it isn't a good general library.
#	You probably don't want to use it.
#
------------------------------------------------------------------------------}

PROGRAM HourGlass;

	USES
		{$LOAD MacIntf.LOAD}
		MemTypes, QuickDraw, OSIntf, ToolIntf, PackIntf,

		{$LOAD HourGlass.LOAD}
		SANE, OneDialog, MIDI;
		
	

	CONST
		DialogRes		= 500;

		{ Dialog Item Numbers }
		NullItem			= 0;
		curTimeItem			= 2;
		quitItem			= 3;
		milliRB				= 4;
		beatRB				= 5;
		fps24RB				= 6;
		fps25RB				= 7;
		fps30DRB			= 8;
		fps30RB				= 9;
		syncInt				= 11;
		syncExt				= 12;
		startbutton			= 13;
		stopbutton			= 14;
		zerobutton			= 15;
		
		updatePeriod		= 3;
		
		myIcon				= 128;
	
	TYPE 
		timeRec = Record
			hours: integer;
			minutes: integer;
			seconds: integer;
			qframes: integer;
		end;
		
	VAR	
		gFormat, gSync : integer;
		curFormat, curSync : integer;
		myrefnum : integer;
		updateTime : boolean;
		gcurTime : longint;
		
	PROCEDURE SaveAwayA5;			EXTERNAL;
	FUNCTION OurA5: LONGINT;		EXTERNAL;
	PROCEDURE TheirA5(a5: LONGINT);	EXTERNAL;
	PROCEDURE Extract(time: longint; var pieces: timeRec); EXTERNAL;
		
	PROCEDURE mytimeproc(time : longint; refcon : longint);
		VAR		oldA5 : longint;

		BEGIN
			oldA5 := OurA5;
			gcurTime := time;
			updateTime := TRUE;
			TheirA5(oldA5);
		END;
		
	PROCEDURE doUpdateTime;
		VAR	
			newTime : longint;
			high,lo : integer;
			trec : timeRec;
			h,m,s,qf : STR255;
		
		BEGIN
			IF	gFormat <= midiFormatBeats THEN
			BEGIN
				s := '';
				message(s,curtimeItem);
				NumToString(gcurTime,s);
				message(s,curtimeItem);
			END
			ELSE
				BEGIN
					newTime := MIDIConvertTime(gFormat, gFormat+(midiFormat24fpsQF-midiFormat24fpsBit), gcurTime);
					Extract(newTime,trec);
					NumToString(trec.hours,h);
					NumToString(trec.minutes,m);
					NumToString(trec.seconds,s);
					NumToString(trec.qframes,qf);
					message(Concat(h,':',m,':',s,'.',qf),curTimeItem);
				END;
			updateTime := FALSE;
		END;
	
		
 
	PROCEDURE StartDialog;
		VAR
				initStuff : MIDIPortParams;
				err : OSErr;
				p : MIDIPortParamsPtr;v : integer;
	
		BEGIN
			TheDialog := GetNewDialog(DialogRes,NIL,Pointer(-1));
			SetPort(TheDialog);
			SetValue(1,v,milliRB);
			curFormat := milliRB;
			SetValue(1,v,syncInt);
			gFormat := midiFormatMSec;
			curFormat := milliRB;
			gSync := midiInternalSync;

			IF SndDispVersion(midiToolNum) <> 0 THEN
				Message('MIDI Manager isn''t there!',curTimeItem);
				{ This should really quit gracefully here
					and not call any other MIDI Manager Functions. }

			err := MIDISignIn('HGls',0,
							GetResource('ICN#', myIcon),
							'Hour Glass');
			if err <> 0 then Debugger;
			WITH initStuff DO
				BEGIN
					portID := 'time';
					PortType := midiPortTypeTime;
					timeBase := 0;
					offsetTime := 0;
					readHook := nil;
					refCon := 0;
					initClock.sync := gSync;
					initClock.curTime := 0;
					initClock.format := gFormat;
					name := 'Time';

				END;
			p := @initStuff;
			err := MIDIAddPort('HGls',0,myrefnum,p);
			if err <> 0 then Debugger;
			MIDIWakeUp(myrefnum,0,2,@mytimeproc);
			
		END;
			

	PROCEDURE RunDialog;
		VAR
			anEvent		: EventRecord;
			v,itemHit	: INTEGER;
			done 		: BOOLEAN;
			whoCares	: BOOLEAN;
			whichWindow	: WindowPtr;
			boundry		: Rect;
			
		BEGIN
			ShowWindow(TheDialog);
			done := FALSE;
			v := 0;
			REPEAT
				IF updateTime THEN doUpdateTime;
				SystemTask;
				whoCares := WaitNextEvent(everyEvent, anEvent, UpdatePeriod, NIL);
				IF IsDialogEvent(anEvent) 
					THEN
						IF DialogSelect(anEvent, TheDialog, itemHit) 
							THEN
								CASE itemHit of
									milliRB:
										BEGIN
											SetValue(0,v,curFormat);
											SetValue(1,v,milliRB);
											IF curFormat <> milliRB THEN
												BEGIN
													curFormat := milliRB;
													MIDISetTCFormat(myrefnum, midiFormatMSec);
													gcurTime := MIDIConvertTime(gFormat,midiFormatMSec,gcurTime);
													updateTime := true;
												END;
											gFormat := midiFormatMSec;
										END;

									beatRB:
										BEGIN
											SetValue(0,v,curFormat);
											SetValue(1,v,beatRB);
											IF curFormat <> beatRB THEN
												BEGIN
													curFormat := beatRB;
													MIDISetTCFormat(myrefnum, midiFormatBeats);
													gcurTime := MIDIConvertTime(gFormat,midiFormatBeats,gcurTime);
													updateTime := true;
												END;	
											gFormat := midiFormatBeats;
										END;
										
									fps24RB:
										BEGIN
											SetValue(0,v,curFormat);
											SetValue(1,v,fps24RB);
											IF curFormat <> fps24RB THEN
												BEGIN
													curFormat := fps24RB;
													MIDISetTCFormat(myrefnum, midiFormat24fpsBit);
													gcurTime := MIDIConvertTime(gFormat,midiFormat24fpsBit,gcurTime);
													updateTime := true;
												END;	
											gFormat := midiFormat24fpsBit;
										END;

									fps25RB:
										BEGIN
											SetValue(0,v,curFormat);
											SetValue(1,v,fps25RB);
											IF curFormat <> fps25RB THEN
												BEGIN
													curFormat := fps25RB;
													MIDISetTCFormat(myrefnum, midiFormat25fpsBit);
													gcurTime := MIDIConvertTime(gFormat,midiFormat25fpsBit,gcurTime);
													updateTime := true;
												END;	
											gFormat := midiFormat25fpsBit;
										END;
										
									fps30DRB:
										BEGIN
											SetValue(0,v,curFormat);
											SetValue(1,v,fps30DRB);
											IF curFormat <> fps30DRB THEN
												BEGIN
													curFormat := fps30DRB;
													MIDISetTCFormat(myrefnum, midiFormat30fpsDBit);
													gcurTime := MIDIConvertTime(gFormat,midiFormat30fpsDBit,gcurTime);
													updateTime := true;
												END;	
											gFormat := midiFormat30fpsDBit;
										END;
										
									fps30RB:
										BEGIN
											SetValue(0,v,curFormat);
											SetValue(1,v,fps30RB);
											IF curFormat <> fps30RB THEN
												BEGIN
													curFormat := fps30RB;
													MIDISetTCFormat(myrefnum, midiFormat30fpsBit);
													gcurTime := MIDIConvertTime(gFormat,midiFormat30fpsBit,gcurTime);
													updateTime := true;
												END;	
											gFormat := midiFormat30fpsBit;
										END;
										
									syncInt:
										BEGIN
											SetValue(1,v,syncInt);
											SetValue(0,v,syncExt);
											gSync := midiInternalSync;
											MIDISetSync(myrefnum,midiInternalSync);
										END;
										
									syncExt:
										BEGIN
											SetValue(1,v,syncExt);
											SetValue(0,v,syncInt);
											gSync := midiExternalSync;
											MIDISetSync(myrefnum,midiExternalSync);
										END;
										
									startbutton:
											MIDIStartTime(myrefnum);
										
									stopbutton:
											MIDIStopTime(myrefnum);
									
									zerobutton:
										MIDISetCurTime(myrefnum,0);
									
									quitItem:
											done := TRUE;
											

									
								END 
							ELSE BEGIN END
						ELSE
							CASE FindWindow(anEvent.where, whichWindow) OF
								inGoAway:
									IF	TrackGoAway(whichWindow, anEvent.where) THEN
										done := TRUE;
						
								inDrag:
									BEGIN
										WITH screenBits.bounds DO
											SetRect(boundry, 4, 24, right - 4, bottom - 4);
											DragWindow(whichWindow, anEvent.where, boundry);
									END;
							END;
				UNTIL done;
			END;
								
		
		PROCEDURE StopDialog;
			BEGIN
				MIDISignOut('HGls');
				HideWindow(TheDialog);
				DisposDialog(TheDialog);
			END;
								


	PROCEDURE InitThings;
		BEGIN
			SaveAwayA5;
			InitGraf(@thePort);
			InitFonts;
			InitWindows;
			InitMenus;
			TEInit;
			InitDialogs(NIL);
			FlushEvents(everyEvent, 0);
			InitCursor;
		END;

	BEGIN
		InitThings;
		StartDialog;
		RunDialog;
		StopDialog;
	END.