{------------------------------------------------------------------------------
#
#	OneDialog
#		- a unit of utilites for programs with one dialog
#
#	OneDialog.p	-	Pascal Source
#
#	Copyright  1987-1989 Apple Computer, Inc.
#	All rights reserved.
#
#	This unit provides a bunch of utilites for manipulating a
#	dialog.  The basic premis is that the program consists of
#	one dialog which controls it.  It is up to the program to
#	load the dialog, set up initial conditions, run the dialog
#	via ModalDialog, interpret item hits, and gracefully shut
#	down.
#
#	This unit is mearly here to support some of the other example
#	applications.  It is not meant as an example of good Macintosh
#	code.  It is NOT a template. It is NOT intended to be 
#	used as a foundation for the next world-class, best-selling, 
#	600K application. A stick figure drawing of the human body may 
#	be a good example of the form for a painting, but that does not 
#	mean it should be used as the basis for the next Mona Lisa.
#
#
------------------------------------------------------------------------------}

UNIT OneDialog;

INTERFACE

	USES
		Memtypes,
		QuickDraw,
		OSIntf,
		ToolIntf;



	VAR
		TheDialog		: DialogPtr;		{The one and only dialog}


{ Initialize Items }
	PROCEDURE FixUserLine(itemNo: INTEGER);
				{Set up a user item to be filled with grey.  If the
				item is only one pixel wide or tall this will produce
				decorative lines in the dialog}
				
	PROCEDURE FixScrollbar(itemNo: INTEGER;
							arrowAmount, pageAmount: INTEGER);
				{Set up scroll bar to do scrolling itself with the
				indicated scrollng amounts.  Use DetermineValue to
				find out the value of the scrollbar after ModalDialog
				returns a hit on it, the scrollbar will do the tracking.}

	PROCEDURE SetIHandle(itemNo: INTEGER; hand: Handle);
				{Set the handle of an item}

{ State Inquirey and Altering }
	PROCEDURE ActivateItem(itemNo: INTEGER);
				{Make a control active}

	PROCEDURE DeactivateItem(itemNo: INTEGER);
				{Make a control inactive}

	PROCEDURE Message(text:Str255; itemNo:INTEGER);
				{Set the text of a static or editable text item}

	PROCEDURE DetermineValue(VAR variable: INTEGER; itemNo: INTEGER);
				{Set variable to be the value of a scrollbar}

	PROCEDURE SetValue(value: INTEGER; VAR variable: INTEGER; itemNo: INTEGER);
				{Set a scrollbar and variable to be a particular value}

	PROCEDURE ChangeState(state:BOOLEAN; VAR variable:BOOLEAN; itemNo:INTEGER);
				{Set a button and variable to a particular state}
	


IMPLEMENTATION

	{$I OneDialog.impl.p}
	
END.

