/*
	File:	SquidLib.h
	Usage:	Header file for SquidLib.c
	Revision History:
		12/23/88 - MIDI Ver 00.50P
			Original.
		01/10/89 -  MIDI Ver 1.0b2 MIDIDriver Ver 1.0b1
			Removed noTempoMap constant.
			Removed tempo map parameters from the declarations of
			AddTimePort() and AddDataPort().
			
	
*/

#ifndef __SquidLib__
#define __SquidLib__

#include "VecaStdLib.h"

typedef struct {
	OSType	ID;
	long	RefCon;
	char	Name[256];
	short	TimeRef;
	char	TimeName[32];
	short	DataInRef;
	char	DataInName[32];
	short	DataOutRef;
	char	DataOutName[32];
} Client, *ClientPtr;

typedef struct {
	unsigned char	Flags;
	unsigned char	Len;
	long			TStamp;
	unsigned char	Status;
	unsigned char	KeyNum;
	unsigned char	Velocity;
} KeyOnPacket;

#define keyOnPacketSize		9 	/* sizeof() padds for even bounds! */


#define squidLibErr			1
#define refConC0			0L
#define refConC1			1L
#define refConC2			2L
#define refConC3			3L
#define refConC4			4L
#define refConP0			0L
#define refConP1			1L
#define refConP2			2L
#define refConP3			3L
#define refConP4			4L
#define timePortBufSize		0
#define inPortBufSize		4096		/* Try This! 65535 */
#define outPortBufSize		0
#define noTimeBase			0
#define noReadHook			0L
#define noPeriod			0L
#define minTime				0x80000000
#define zeroTime			0x00000000
#define zeroPeriod			0x00000000
#define maxTime				0x7FFFFFFF
#define noClientID			('    ')
#define AppleMIDIDriverID	('amdr')
#define AppleTimePortID		('ATim')
#define AppleMIDIInID		('Ain ')
#define AppleMIDIOutID		('Aout')

extern OSErr	CalamariErr;
extern FILE 	*MyErr;

/*
	To Avoid Str255's Use This!
*/
OSErr
	midiSignIn(OSType TheClientID, long TheRefCon, Handle TheIcon, char *TheClientName);


/* 
	Convert MIDIMgr SndDispVersion # from an unsigned 16-bit 
	integer to a SndDispVersion string.
*/
extern char *
	SquidVer2String(Uword N, char *str);

/* 
	Adds a TIME Port to TheClient, 
	and returns the port's reference number.
*/
extern short	
	AddTimePort(OSType		TheClientID, 
				OSType		ThePortID, 
				short		ThePortType,
				short		TheTimeBase,
				long		TheOffsetTime,
				Ptr			TheReadHook,
				long		TheRefcon,
				char		TheName[],
				short		TheSyncMode,
				long		TheCurTime,
				short		TheTimeFormat
				);

/* 
	Adds a Data Port with SquidLib Default Input 
	Buffer Size to TheClient, 
	and returns the port's reference number.
*/	
extern short	
	AddDataPort(OSType		TheClientID, 
				OSType		ThePortID, 
				short		ThePortType,
				short		TheTimeBase,
				long		TheOffsetTime,
				Ptr			TheReadHook,
				long		TheRefcon,
				char		TheName[]
				);
				
/* 
	Adds an Input Port with Given Input Buffer Size to TheClient, 
	and returns the port's reference number.
	NOTE: this funtion may set CalamariErr and return squidLibErr.
*/
extern short		
AddInputPort(OSType		TheClientID, 
			OSType		ThePortID, 
			short		ThePortType,
			short		TheTimeBase,
			long		TheOffsetTime,
			Ptr			TheReadHook,
			long		TheRefcon,
			char		TheName[],
			short		BufSize
			);

				
/*
	Logs all current Client and Port information to the log file 
	pointed to by MyErr (externally defined FILE *).
*/
extern void
	LogAllClientInfo(void);

	
/*
	Convert a port type (short int) to a printable string.
*/
extern char *
	PortType2String(short TypeNum, char *CStrBuf);
	

/*
	Log a MIDIMgr function return error code to the log file 
	pointed to by MyErr (externally defined FILE *).
*/	
extern OSErr
	ReportOSErr(OSErr TheErr, char *TheMessage);

/*
	Convert a defined MIDI error code to a string.
*/
extern char *
	CalamariErrToStr(OSErr TheErr, char *TheStrBuf);

/*
	Return the current MIDI Error Code in English.
*/	
extern char *
	CalamariErrString(char *TheStrBuf);

/*
	Convert a defined MIDI Time Code Format short int into a string.
*/
extern char *
	TCFormatToStr(short TheTCFormat, char *TheStrBuf);
	
/*
	Signs out ALL USER Clients.  
	Does NOT sign out "Patcher" ('pchr') or "MIDIDriver" ('amdr').
*/
extern void
	SignOutAllUserClients(void);

/*
	Send a key on MIDIPacket to the MIDI Manager
*/
extern OSErr
	SendKeyOn(short OutputPortRefNum, long TimeStamp, 
				Byte KeyNum, Byte Velocity);
/*
	Determine if a MIDIPacket's flags indicate a MIDI message or not.
*/
extern Boolean
	IsMIDIMessage(Byte TheFlags);
	
extern OSErr
	AllNotesOff(short PortRefNum);
	
	
/*************** These are too dumb to leave in as of 3/20/89 DV *********/	

/*
	Initializes the given Client Data Structure (typedefed in SquidLib.h) 
	to the given parameters.
*/
extern void
	InitClient(ClientPtr TheClientPtr, 
			   OSType TheID, 
			   long TheRefCon, 
			   char TheName[]);
			  
/*
	Signs in the given client and opens a 
	time, data-in, and data-out port for the client.
*/
extern void
	StartUpClient(ClientPtr TheClientPtr, Ptr TheReadHook);


/*
	Signs Out a given Client.  
	(I don't know why it Call MIDIStopTime() first.)???
*/
extern void
	ShutDownClient(ClientPtr TheClientPtr);
	
/********************** end dumb section ***************************************/	
	
#endif /* define _SquidLib_ */


