/*
	File:	VecaStdLib.c
	Usage:	Simple functions that should have been 
	included in the MPW standard library in the first place!
*/

#include <Types.h>
#include <Events.h>
#include <OSEvents.h>
#include <Resources.h>
#include <Quickdraw.h>
#include <Fonts.h>
#include <Windows.h>
#include <Menus.h>
#include <Desk.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <OSUtils.h>
#include <Controls.h>
#include <Packages.h>
#include <Memory.h>
#include <ToolUtils.h>
#include <Files.h>
#include <Errors.h>
/* #include <Strings.h> */
#include <stdIO.h>
#include <stdlib.h>
#include "VecaStdLib.h"


/*
	Global File Pointers for Log File Output.
*/
FILE	*MyErr, *MyLog;


/*
	Convert a String to an Integer 
*/
int
StrToInt(char s[])
{
	int i;
	int n = 0;
	
	for (i = 0; s[i] >= '0' && s[i] <= '9'; i++) {
		n = 10 * n + s[i] - '0';
	}
	
	return(n);
}

/*
	Convert a String to an short. 
*/
short
StrToShort(char s[])
{
	int i;
	short n = 0;
	
	for (i = 0; s[i] >= '0' && s[i] <= '9'; i++) {
		n = 10 * n + s[i] - '0';
	}
	
	return(n);
}


/*
	Convert a String to an long. 
*/
long
StrToLong(char s[])
{
	int i;
	long n = 0;
	//char CStrBuf1[256];
	
	//sprintf(CStrBuf1,"Enter StrToLong( %s )", s);
	//StdAlert(CStrBuf1);
	
	for (i = 0; s[i] >= '0' && s[i] <= '9'; i++) {
		n = 10 * n + s[i] - '0';
	}
	
	//sprintf(CStrBuf1,"Exit StrToLong( %s ), returning %ld", s, n);
	//StdAlert(CStrBuf1);
	
	return(n);
}

char *
ShortToStr(n, s)
short n;
char s[];
{
	short i = 0, sign;
	
	if ((sign = n) < 0)	/* record sign */
		n = -n;
		
	do {
		s[i++] = n % 10 + '0';
	} while ((n /= 10) > 0);
	
	if (sign < 0)
		s[i++] = '-';
	s[i] = '\0';
	
	return( ReverseStr(s) );
}

char *
IntToStr(n, s)
int n;
char s[];
{
	int i = 0, sign;
	
	if ((sign = n) < 0)	/* record sign */
		n = -n;
		
	do {
		s[i++] = n % 10 + '0';
	} while ((n /= 10) > 0);
	
	if (sign < 0)
		s[i++] = '-';
	s[i] = '\0';
	
	return( ReverseStr(s) );
}

char *
LongToStr(n, s)
long n;
char s[];
{
	int i = 0, sign;
	
	if ((sign = n) < 0)	/* record sign */
		n = -n;
		
	do {
		s[i++] = n % 10 + '0';
	} while ((n /= 10) > 0);
	
	if (sign < 0)
		s[i++] = '-';
	s[i] = '\0';
	
	return( ReverseStr(s) );
}


char *
ReverseStr(s)
char s[];
{
	int c, i, j;
	
	for (i=0, j=strlen(s)-1; i<j; i++, j--) {
		c = s[i];
		s[i] = s[j];
		s[j] = c;
	}
	
	return(s);
}

int StrLen(s)
char *s;
{
	int n;
	
	for (n=0; *s != '\0'; s++)
		n++;
	return(n);
}
	

char *
P2CStr(str)
char *str;
{
	short i, len = str[0];
	
	for (i=0; i<len; i++) {
		str[i] = str[i+1];
	}
	str[i] = 0;
	
	return( str );
}

char *
C2PStr(str)
char *str;
{
	short i, len = strlen(str);

	
	for(i=len; i>0; i--) {
		str[i] = str[i-1];
	}
	str[i] = len;
	
	return( str );
}

/*
	Just like standard Strcpy(), but from (Source, Dest) instead.
*/
char *
StrCpy(char *SourceStr, char *DestStr)
{
	short i;
	
	
	for(i=0; SourceStr[i] != '\0'; i++) {
		DestStr[i] = SourceStr[i];
	}
	DestStr[i] = '\0';
	
	return(DestStr);
}

/*
	Just like  StrCpy(), but with Pascal strings instead.
*/
char *
Pstrcpy(Str255 Pstr, char *buf)
{
	short i, len = Pstr[0];
	
	buf[0] = len;
	for(i=1; i<=len; i++) {
		buf[i] = Pstr[i];
	}
	
	return(buf);
}



/*
	Convert a C String (from Cstr) into a Pascal string, 
	copying the result into a buffer (Pstr).
*/
char *
C2PStrCpy(char *Cstr, Str255 Pstr)
{
	short i, len = strlen(Cstr);
	
	for(i=len; i>0; i--) {
		Pstr[i] = Cstr[i-1];
	}
	Pstr[i] = len;
	
	return( (char *) Pstr );
}


/*
	Convert a Pascal String (from Pstr) into a C string, 
	copying the result into a buffer (Cstr).
*/
char *
P2CStrCpy(Str255 Pstr, char *Cstr)
{
	short i, len = Pstr[0];
	
	for (i=0; i<len; i++) {
		Cstr[i] = Pstr[i+1];
	}
	Cstr[i] = '\0';
	
	return( Cstr );
}

/*
	Determine if the string arg is a NUMBER or not.
	NOTE:  As of 01/30/89 it only looks for Integers.
*/
Boolean
IsNumStr(char *TheStringBuf)
{
	Boolean RetVal = true;
	
	while( *TheStringBuf != '\0' ) {
		if (  !((*TheStringBuf >= '0') && (*TheStringBuf <= '9')) ) { 
			RetVal = false;
			break;
		}
		else {
			TheStringBuf++;
		}
	}
	
	return(RetVal);
}

Boolean 
HexDump(Uchar *FromHere, long Len, FILE *fp)
{

	long	i, j;
	long	Rows = Len/4;
	Uword	*WordPtr = (Uword *)  FromHere;
	Uchar	*BytePtr = (Uchar  *) FromHere;

	fprintf(fp,"\n\n->ENTER HexDump() : Len = %ld\n\n", Len);

	for (i=0; i<Rows; i++) {
	
		fprintf(fp,"%04X %04X %04X %04X \t\t", *(WordPtr+0), *(WordPtr+1), *(WordPtr+2), *(WordPtr+3));
		
		for (j=0; j<8; j++) {
		/*
			fprintf(fp, "%c",  *(BytePtr + j) );
		*/
			fprintf(fp, "%c", IsPrintable(*(BytePtr + j)) ? *(BytePtr + j) : '.');
		}
		
		fprintf(fp,"\n");
		
		WordPtr+=4;
		BytePtr +=8;
	}
		
	fprintf(fp,"\n\n<-EXIT HexDump()\n\n");
	return(true);
}
		
	
Boolean IsPrintable(TheChar)
Uchar TheChar;
{
	return( (TheChar>31 && TheChar<127) || (TheChar > 127) );
}
	
	

/*** Convert version # from an unsigned long to a version string */
char *
VerLong2String(Ulong N, char *str)
{
	short	MajorVer = (short) ((N >> 16) & 0x00007FFF);
	short	MinorVer = (short) (N & 0x0000FFFF);
	Boolean	Prototype = (N & 0x80000000) != 0;
	
	sprintf(str,"%02d.%02d%c", MajorVer, MinorVer, Prototype ? 'P' : '');
	
	return(str);
}
	
/*** Convert version # from an unsigned 16-bit integer to a version string */
char *
VerShort2String(Uword N, char *str)
{
	short	MajorVer =	((N >> 8) & 0x007F);
	short	MinorVer =	(N & 0x00FF);
	Boolean	Prototype = (N & 0x8000) != 0;
	
	sprintf(str,"%02d.%02d%c", MajorVer, MinorVer, Prototype ? 'P' : '');
	
	return(str);
}

Rect *
RectCopy(SourceRectPtr, DestRectPtr)
Rect *SourceRectPtr;
Rect *DestRectPtr;
{
	DestRectPtr->top 	= SourceRectPtr->top;
	DestRectPtr->left 	= SourceRectPtr->left;
	DestRectPtr->bottom = SourceRectPtr->bottom;
	DestRectPtr->right 	= SourceRectPtr->right;
	
	return( DestRectPtr );
}

/*** CONVERTS AN OSType TO A STRING REPRESENATION ***/
char *
OSType2String(OSType N, char *Buf)
{
	Buf[0] = '\'';
	Buf[1] = (char) ((N & 0xFF000000) >> 24);
	Buf[2] = (char) ((N & 0x00FF0000) >> 16);
	Buf[3] = (char) ((N & 0x0000FF00) >> 8);
	Buf[4] = (char) ((N & 0x000000FF) >> 0);
	Buf[5] = '\'';
	Buf[6] = '\0';

	return(Buf);
}


/*
	Stupid delay.
*/
void
MyDelay(short n)
{
	short i, j;
	
	for (i=0; i<n; i++) {
		for(j=0; j<10000; j++);
	}
	
}


short 
Seed(void)
{
	unsigned long secs;
	
	GetDateTime(&secs);
	return( qd.randSeed = secs);
	
}
/*
	Returns positive random result between 0 and max-1 inclusive.
*/
short 
Choose(short max)
{
	long RawResult = labs( (long) Random() );
	
	return( (short) ((RawResult*max) / 32768) );
}

#define	TrueOrFalse()	Choose(2)

/*
	Returns positive or negative random result between 0 and max-1 inclusive.
*/	
short 
SChoose(short max)
{
	long RawResult = Random();
	
	return( (short) ((RawResult*max) / 32768) );
}

/*
	Returns random result between Lo and Hi inclusive.
*/
short 
Rnd(short Lo, short Hi)
{
	return( Choose(Hi - Lo + 1) + Lo);
}



/*************************** Dialog Jizz **********************/
/*	
	NOTE:  All of the following dialog routine names 
	begin with "Std" 
*/

/*
	Print out an Alert message to the user.
*/
void
StdAlert(char *TheMessage)
{
	paramtext(TheMessage, "","","");
	Alert(stdAlertBoxID, NULL);
}

/*
	Print out a dialog message to the user, and wait until the user 
	clicks the OK button to continue the program.
*/
void
StdWaitForContinue(char *Message)
{
	short		ItemHit, Done = false;
	DialogPtr	TheDialog;
	EventRecord AnEvent;
	
	TheDialog = GetNewDialog(stdContinueID, NULL, (WindowPtr) -1);
	SetPort(TheDialog); 
	StdSetText(Message, TheDialog, stdContinueStatusText1ID);
	
	StdHiliteButton(TheDialog, stdContinueOKButtonID);
	
	ShowWindow(TheDialog);
	
	while(!Done) {
		if ( WaitNextEvent(everyEvent, &AnEvent,stdUpdatePeriod, NULL) ) {
			StdHiliteButton(TheDialog, stdContinueOKButtonID);
			if ( IsDialogEvent(&AnEvent) ) {
				if (AnEvent.what == keyDown 
					&& ( (AnEvent.message & charCodeMask) == charEnterKey )) 
				{
					Done = true;
				}
				else if ( DialogSelect(&AnEvent, &TheDialog, &ItemHit) ) {
					if (ItemHit == stdContinueOKButtonID) {
						Done = true;
					}
				}
			}
		}
	}
					
	HideWindow( (WindowPtr) TheDialog);
	DisposDialog(TheDialog);
	
}

/* 
	Boldface a button. 
*/
void
StdHiliteButton(DialogPtr TheDialog, short TheItemID)
{
	short		ItemType;
	Handle		ItemHndl;
	Rect		ItemBox;
	
	GetDItem(TheDialog, TheItemID, &ItemType, &ItemHndl, &ItemBox);
	PenSize(3,3);
	InsetRect(&ItemBox,-4,-4);
	FrameRoundRect(&ItemBox,16,16);	
}

/*
 * Set the text in a static text item of a dialog box.
 */
void 
StdSetText(char *text, DialogPtr dialog, short itemNo)
{
	short	itemType;
	Handle	itemHndl;
	Rect	itemBox;    
	
	GetDItem(dialog, itemNo, &itemType, &itemHndl, &itemBox);
	setitext (itemHndl,text);
}


/*
	Get the text from an editText item.
*/
char *
StdGetEditTextStr(char *TheString, DialogPtr TheDialog, short TheItemNum)
{
	char 	*RetVal = NULL;
	short	ItemType;
	Handle	ItemHndl;
	Rect	ItemBox;
	//char	CStrBuf1[256];
	
	//StdAlert("Enter: StdGetEditTextStr()");
	
	GetDItem(TheDialog, TheItemNum, &ItemType, &ItemHndl, &ItemBox);	/* get the item */
	
	if (ItemHndl != NULL) {		/* lock it down */
		HLock(ItemHndl);
	}
	else {
		StdAlert("VecaLib ERROR in StdGetEditTextStr(): GetDItem() - (ItemHndl == NULL)");
		ExitToShell();
	}
	
	getitext(ItemHndl, TheString);		/* get the String */
	HUnlock(ItemHndl);					/* unlock item */
	RetVal = TheString;					/* for futur error checking */
	
	//sprintf(CStrBuf1,"Exit StdGetEditTextStr(): RetVal = %s", RetVal);
	//StdAlert(CStrBuf1);
	
	return(RetVal);						/* return the pointer to the String */									
}


/*
	Get an integer from the EditText Item of a dialog.
*/
int
StdGetEditTextInt(DialogPtr TheDialog, short TheItemNum)
{
	char StrBuf[256];
	
	return( StrToInt( StdGetEditTextStr(StrBuf, TheDialog, TheItemNum) ) );
}

/*
	Get a short integer from the EditText Item of a dialog.
*/
short
StdGetEditTextShort(DialogPtr TheDialog, short TheItemNum)
{
	char StrBuf[256];
	
	return( StrToShort( StdGetEditTextStr(StrBuf, TheDialog, TheItemNum) ) );
}

/*
	Get a long integer from the EditText Item of a dialog.
*/
long
StdGetEditTextLong(DialogPtr TheDialog, short TheItemNum)
{
	char StrBuf[256]; 
	long RetVal;
	//char CStrBuf1[256];
	
	//sprintf(CStrBuf1,"Enter StdGetEditTextLong(): TheItemNum = %d", TheItemNum);
	//StdAlert(CStrBuf1);
	
	RetVal = StrToLong( StdGetEditTextStr(StrBuf, TheDialog, TheItemNum) );
	
	//sprintf(CStrBuf1,"EXIT StdGetEditTextLong(): returning = %ld", RetVal);
	//StdAlert(CStrBuf1);
	
	return( RetVal );
}

long 
StdGetNumLong(char *ThePrompt, char *DefaultNumStr)
{
	short 		ItemHit, Done = false;
	DialogPtr 	TheDialog;
	EventRecord AnEvent;
	char		TheStringBuf[64];
	long		TheReturnValue;
	
	TheDialog = GetNewDialog(stdGetNumDialogID, NULL, (WindowPtr) -1);
	SetPort(TheDialog); 
	
	StdSetText(ThePrompt, TheDialog, stdGetNumPromptID);
	StdSetText(DefaultNumStr, TheDialog, stdGetNumUserInputID);
	
	/* Highlight the ENTIRE default input text */
	SelIText(TheDialog, stdGetNumUserInputID, startOfSelText, maxEndOfSelText);
	/* Highlight the OK button */
	StdHiliteButton(TheDialog, stdGetNumOKID);
	
	ShowWindow(TheDialog);
	
	while(!Done) {
		TEIdle( ((DialogPeek)TheDialog)->textH );
		if ( WaitNextEvent(everyEvent, &AnEvent, stdUpdatePeriod, NULL) ) {
			StdHiliteButton(TheDialog, stdGetNumOKID);
			if ( IsDialogEvent(&AnEvent) ) {
				if ((AnEvent.what == keyDown) && 
					((AnEvent.message & charCodeMask) == charEnterKey)) 
				{
					Done = true;
					StdGetEditTextStr(TheStringBuf, TheDialog, stdGetNumUserInputID);
					if ( !IsNumStr(TheStringBuf) ) {
						StdSetText("Please Enter A NUMBER Damn-It!", 
									TheDialog, stdGetNumPromptID);
						Done = false;
					}
				}
				else if ( DialogSelect(&AnEvent, &TheDialog, &ItemHit) ) {
					if (ItemHit == stdGetNumOKID) {
						Done = true;
						StdGetEditTextStr(TheStringBuf, TheDialog, stdGetNumUserInputID);
						if ( !IsNumStr(TheStringBuf) ) {
							StdSetText("Please Enter A NUMBER Damn-It!", 
										TheDialog, stdGetNumPromptID);
							Done = false;
						}
					}
				}
			}
		}
	}
			
	TheReturnValue = StdGetEditTextLong(TheDialog, stdGetNumUserInputID);
	
	HideWindow( (WindowPtr) TheDialog);
	DisposDialog(TheDialog);
	
	return(TheReturnValue);
	
}

	
void 
StdSwitchRadio(DialogPtr TheDialog, short *CurRadio, short NextRadio)
{
	StdChangeState (TheDialog, 0, *CurRadio);
	StdChangeState (TheDialog, 1, NextRadio);
	*CurRadio = NextRadio;
		
}

/*
	Set the state of a binary control, such as a radio button or
 	checkbox.
*/
void StdChangeState(DialogPtr TheDialog, short State, short ItemNo)
{
	short	ItemType;
	Handle	ItemHndl;
	Rect	ItemBox;
	
	GetDItem(TheDialog, ItemNo, &ItemType, &ItemHndl, &ItemBox);
	SetCtlValue ( (ControlHandle) ItemHndl, (State ? 1 : 0) );
}

/*
	Save the Coordinates of the Dialog box into 
	it's resource file; i.e., when box is closed, we open it in that same place.
*/
void
StdSaveDLOGLocation(DialogPtr TheDialogPtr, short TheDialogID)
{
	Handle	MyDLOGResHndl;
	Rect	WhereItIs;
	
	MyDLOGResHndl = GetResource('DLOG', TheDialogID);
	if (MyDLOGResHndl != NULL) {
		HNoPurge(MyDLOGResHndl);
		SetPort(TheDialogPtr);
		WhereItIs = TheDialogPtr->portRect;
		LocalToGlobal( (Point *) &(WhereItIs.top) );
		LocalToGlobal( (Point *) &(WhereItIs.bottom) );
		( *( (DialogTHndl) MyDLOGResHndl ) )->boundsRect = WhereItIs;
		ChangedResource(MyDLOGResHndl);
		WriteResource(MyDLOGResHndl);
		HPurge(MyDLOGResHndl);
	}
	else {
		StdAlert("Error in StdSaveDLOGLocation():  MyDLOGResHndl = NULL!");
		Terminate();
	}
	
}

/*
	If Dialog is off screen, then move it back on.
*/
void
StdAdjustDLOGLocation(DialogPtr TheDialogPtr)
{
	Point	TestPoint;
	
	TestPoint.h = TheDialogPtr->portRect.left + 6;	/* 6 = close enough to grab */
	TestPoint.v = TheDialogPtr->portRect.top + 6;	/* 6 = close enough to grab */
	LocalToGlobal(&TestPoint);
	
	if ( !PtInRgn( TestPoint, GetGrayRgn() ) ) {	/* Reposition */
		MoveWindow( (WindowPtr) TheDialogPtr, 
			qd.screenBits.bounds.left + 50,			
			qd.screenBits.bounds.top  + 50,
			false
		);
	}
	
}


/*_______________________________________________________________________________________*/
/*___________________________ Window Jizz _______________________________________________*/

Boolean 
StdIsDAWindow(WindowPtr window)
{
	if ( window == nil )
		return false;
	else	/* DA windows have negative windowKinds */
		return ((WindowPeek) window)->windowKind < 0;
} /*IsDAWindow*/

Boolean 
StdIsAppWindow(window)
	WindowPtr	window;
{
	if ( window == nil )
		return false;
	else	/* application windows have non-negative windowKinds */
		return ((WindowPeek) window)->windowKind >= 0;
} /*IsAppWindow*/

/*_______________________________________________________________________________________*/
	
	
void
StdOpenLogFile(char *FileName)
{
	char CStrBuf1[256];
	
	if ( !(MyLog = fopen(FileName,"w")) ) {
		sprintf(CStrBuf1,"Can't open log file %s for write!  Aborting...\n", FileName);
		StdAlert(CStrBuf1);
		ExitToShell();
	}
}

void
StdCloseLogFile(void)
{
	fclose(MyLog);
}

void
StdLogMsg(char *TheMessage)
{
	fprintf(MyLog,"%s", TheMessage);
}
