/*
	File:	VecaStdLib.h
	Usage:	Header file for VecaStdLib.c
	
*/

#ifndef __VecaStdLib__
#define __VecaStdLib__

#include <stdio.h>
#include <stdlib.h>
#include <Types.h>
#include <Dialogs.h>


typedef unsigned char	Uchar;
typedef unsigned char	Ubyte;
/*	typedef unsigned char	Byte;									*/
typedef unsigned short	Uword;
typedef unsigned long	Ulong;

#define stdUpdatePeriod		3

#define charEnterKey		0x03
#define keyEnterKey1		0x34
#define keyEnterKey2		0x4C
#define startOfSelText		0
#define maxEndOfSelText		32767
#define maxSWord			32767
#define maxUWord			65535

/*
	STANDARD MENU JIZZ 
*/
#define	stdMenuBar			128		/* Application's menu bar ID */
#define	stdAppleMenu		128		/* Apple Menu */
#define stdAboutItem		1
#define stdFileMenu			129		/* File Menu */
#define	stdCloseItem		4
#define stdQuitItem			12
#define	stdEditMenu			130		/* Edit Menu */
#define	stdUndoItem			1
#define	stdCutItem			3
#define	stdCopyItem			4
#define	stdPasteItem		5
#define	stdClearItem		6


/* 
	Standard Dialog Resource ID's 
*/
/*** Standard 'User Message - Wait until OK click - Continue' dialog box ***/ 
#define stdContinueID		1000	
/* ItemList ID's for Continue Dialog Box Resource */
#define stdContinueOKButtonID		1
#define stdContinueStatusText1ID 	2

/*** Standard Interrupt Level Status Display Dialog Box ***/ 
#define stdInterruptDialogID		1001	
/* ItemList ID's for Interrupt Level Status Display Dialog Box Resource */
#define stdInterruptOKButtonID	1
#define stdInterruptStatus1ID	2
#define stdInterruptStatus2ID 	3

/*** Standard Get User Number Dialog Box ***/ 
#define stdGetNumDialogID		1002	
/* ItemList ID's for Get User Number Dialog Box Resource */
#define stdGetNumOKID			1
#define stdGetNumPromptID		2
#define stdGetNumUserInputID	3	



/*** Standard Alert Box ***/
#define stdAlertBoxID		12345


/*
	Global File Pointers for Log File Output.
*/
extern FILE	*MyErr, *MyLog;



/*
	Convert a String to an Integer 
*/
extern int
	StrToInt(char s[]);
	
/*
	Convert a String to an short. 
*/
extern short
	StrToShort(char s[]);

/*
	Convert a String to an long. 
*/	
extern long
	StrToLong(char s[]);

extern char *	
	ShortToStr(short n, char *s);
	
extern char *	
	IntToStr(int n, char *s);
	
extern char *	
	LongToStr(long n, char *s);
	
extern char *	
	ReverseStr(char *s);
	
extern int	 	
	StrLen(char *s);
	
extern char *	
	P2Cstr(char *str);
	
extern char *	
	C2Pstr(char *str);
	
/*
	Just like standard strcpy(), but with Pascal strings instead.
*/
extern char *
	Pstrcpy(char *buf, char *Pstr);
	
/*
	Determine if the string arg is a NUMBER or not.
	NOTE:  As of 01/30/89 it only looks for Integers.
*/
extern Boolean
	IsNumStr(char *TheStringBuf);
	
extern Boolean 
	HexDump(Uchar *FromHere, long Len, FILE *fp);
	
extern Boolean 
	IsPrintable(char TheChar);
	
extern char *
	VerLong2String(Ulong N, char *str);

extern char *
	VerShort2String(Uword N, char *str);

	
extern Rect *	
	RectCopy(Rect *SourceRectPtr, Rect *DestRectPtr);
	
	
extern char *
	OSType2String(OSType N, char *Buf);
		

/*
	Stupid delay.
*/
extern void
	MyDelay(short n);

/*
	Randomizers
*/
extern short 
	Seed(void);

/*
	Returns positive random result between 0 and max-1 inclusive.
*/
extern short 
	Choose(short max);

/*
	Returns positive or negative random result between 0 and max-1 inclusive.
*/	
extern short 
	SChoose(short max);

/*
	Returns random result between Lo and Hi inclusive.
*/
extern short 
	Rnd(short Lo, short Hi);

	
/*
	Convert a C String (from Cstr) into a Pascal string, 
	copying the result into a buffer (Pstr).
*/
extern char *
	C2PStrCpy(char *Cstr, Str255 Pstr);
	
	
/*
	Convert a Pascal String (from Pstr) into a C string, 
	copying the result into a buffer (Cstr).
*/
extern char *
	P2CStrCpy(Str255 Pstr, char *Cstr);
	
	


/*************************** Dialog Jizz **********************/
/*	
	NOTE:  All of the following dialog routine names 
	begin with "Std" 
*/
	
/*
	Print out an Alert message to the user.
*/
extern void
	StdAlert(char *TheMessage);

/*
	Print out a dialog message to the user, and wait until the user 
	clicks the OK button to continue the program.
*/	
extern void
	StdWaitForContinue(char *Message);
	
	
/*
	Set the text in a static text item of a dialog box.
 */
extern void 
	StdSetText(char *text, DialogPtr dialog, short itemNo);
		
/*
	Get the text from an editText item.
*/
extern char *
	StdGetEditTextStr(char *TheString, DialogPtr TheDialog, short TheItemNum);

/*
	Get an integer from the EditText Item of a dialog.
*/
extern int
	StdGetEditTextInt(DialogPtr TheDialog, short TheItemNum);
	
/*
	Get a short integer from the EditText Item of a dialog.
*/
extern short
	StdGetEditTextShort(DialogPtr TheDialog, short TheItemNum);
	
/*
	Get a long integer from the EditText Item of a dialog.
*/
extern long
	StdGetEditTextLong(DialogPtr TheDialog, short TheItemNum);

/*
	Get a long integer from the EditText Item of stdGetNum dialog.
*/
extern long
	StdGetNumLong(char *ThePrompt, char *DefaultNumStr);

/* 
	Boldface a button. 
*/	
extern void
	StdHiliteButton(DialogPtr TheDialog, short TheItemID);
	
/*
	Change the state of a radio button, 
	and set their identifier vars accordingly.
*/
extern void 
	StdSwitchRadio(DialogPtr TheDialog, short *CurRadio, short NextRadio);
	
/*
	Set the state of a binary control, such as a radio button or
 	checkbox.
*/
extern void 
	StdChangeState(DialogPtr TheDialog, short State, short ItemNo);
	
/*
	Save the Coordinates of the Dialog box into 
	it's resource file; i.e., when box is closed, we open it in that same place.
*/
extern void
	StdSaveDLOGLocation(DialogPtr TheDialogPtr, short TheDialogID);

/*
	If Dialog is off screen, then move it back on.
*/
extern void
	StdAdjustDLOGLocation(DialogPtr TheDialogPtr);

/*_______________________________________________________________________________*/
/********************************** Window Jizz ************************************/

extern Boolean 
	StdIsDAWindow(WindowPtr window);

extern Boolean 
	StdIsAppWindow(WindowPtr window);
	
/*_______________________________________________________________________________*/

extern void
	StdOpenLogFile(char *FileName);

extern void
	StdCloseLogFile(void);

extern void
	StdLogMsg(char *TheMessage);
	
#endif