/*
#
#	SquidCakes
#		- a sample MIDI Management Tools application
#
#	SquidCakes.c	-	C Source
#
#	Copyright  1988-1989 Apple Computer, Inc.
#	All rights reserved.
#
#	Version:	1.0
#
#	SquidCakes is a sample application that uses the MIDI Management
#	Tools.  It displays all packets that it gets in hex.  It also can
#	record these packets and play them back with orginal timing.  It
#	is fairly useful as a debugging tool, but not for much else.
#
#	Please be familiar with both the MIDI Management Tools and MIDI
#	in general before trying to understand this code.  
#
#	This application is an example of how to call the MIDI Management
#	tools.  It is not meant as an example of a good Macintosh
#	application; it is NOT a template. It is NOT intended to be 
#	used as a foundation for the next world-class, best-selling, 
#	600K application. A stick figure drawing of the human body may 
#	be a good example of the form for a painting, but that does not 
#	mean it should be used as the basis for the next Mona Lisa.
#
#	This code also makes use of a two libraries, SquidLib & VecaLib.
#	They are small sets of routines that are only useful for very 
#	simple applications.  Again, they aren't good general libraries.
#	You probably don't want to use them.
#	
*/

#include <stdIO.h>
#include <stdlib.h>
#include <Values.h>
#include <Types.h>
#include <Resources.h>
#include <QuickDraw.h>
#include <Fonts.h>
#include <Events.h>
#include <Controls.h>
#include <Windows.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <Desk.h>
#include <Scrap.h>
#include <ToolUtils.h>
#include <Memory.h>
#include <SegLoad.h>
#include <Files.h>
#include <OSUtils.h>
#include <OSEvents.h>
#include <Traps.h>
#include <Errors.h>
#include <Sound.h>
#include <MIDI.h>
#include "MIDIDefs.h"
#include "VecaStdLib.h"
#include "A5Tools.h"
#include "SquidLib.h"

#include "SquidCakes.h"

#define lineSize	12


/*** GLOBAL VARIABLES - All in one big happy structure! ***/
struct {
	
	/* The port reference numbers */
	short		TimeBaseRefNum;		/* Time Base Port */
	short		InPortRefNum;		/* Data Input Port */ 
	short		OutPortRefNum;		/* Data Output Port */
	
	/* Our own data buffer and paraphenalia */
	Uchar		RecordingBuf[recordingBufSize];		
	short		RecordingIndex;		
	short		PlayingIndex;
	Boolean		RecordingNow;
	Boolean		PlayingNow;
	long		FirstTime;			/* Time stamp of first data message in sequence */
	long		FinalTime;			/* Time stamp of last data message in sequence */
	
	long		CurTimeOffset;			/* determines when we 'think' data is ready to process */
	Boolean		FastUpdate;			/* Get as much as we can each time thru loop.*/
	Uchar		TypeLookupStr[256];	/* MIDIPacket Type Lookup Table */
	Uchar		StatLookupStr[256];	/* MIDIPacket Status Lookup Table */
	Rect		DisplayRect;		/* Text display rectangel */
	short		LastSpot;			/* Location of next display */
	Boolean		ShutDownNow;
	Boolean		Done;

	
} G;	

DialogPtr	TheDialog;	/* The one and only dialog box */
char CStrBuf1[256], CStrBuf2[256];


main() 
{
	InitThings();
	StartDialog();
	StartUp();
	RunDialog();
	ShutDown();
	StopDialog();
	CloseThings();
}


void
InitThings(void)
{
	
	FlushEvents(everyEvent, 0);		/* empty any stray events.			*/
	InitGraf(&qd.thePort);			/* Initialize quickdraw.			*/
	InitFonts();					/* Initialize Font Manager.			*/
	InitWindows();					/* Initialize Window Manager.		*/
	InitMenus();					/* Initialize Menu Manager.			*/
	TEInit();						/* Initialize Text Edit.			*/
	InitDialogs(NULL);				/* Initialize the Dialog Manager.	*/
	InitCursor();					/* Turn cursor back to arrow.		*/
	
	{		// Set up a standard menu bar.
		Handle MenuBar = GetNewMBar(menuBar);		// Read menus into menu bar.
		SetMenuBar(MenuBar);						// Install menus.
		DisposHandle(MenuBar);
		AddResMenu( GetMHandle(appleMenu), 'DRVR');	// Add DA names to Apple menu.
		DrawMenuBar();
	}
	
	
	
	StdOpenLogFile("SquidCakes.log");
	
	G.ShutDownNow = false;
	
	
	StdLogMsg("\n\nEXIT-> InitThings()\n\n");
	
}


void
StartDialog(void)			
{

	Boolean		WhoCares;
	EventRecord	AnEvent;
	short		ItemType;
	Handle		ItemHdl;
	Rect		Box;

	
	StdLogMsg("\n\nENTER-> StartDialog()\n\n");
	
	WhoCares = WaitNextEvent(0, &AnEvent, updatePeriod, NULL);

	TheDialog = GetNewDialog(dialogResID, NULL, (WindowPtr) -1);

	SetPort(TheDialog);		
		/* TheDialog is defined in the OneDialog Module */

	MySetIHandle(displayItem, (Handle) UpdateDisplay);	
		/* UpdateDisplay is a Function defined in this module */

	G.CurTimeOffset = midiGetCurrent;	
	MyChangeState(G.CurTimeOffset == midiGetCurrent, &WhoCares, currentItem);

	MyChangeState(G.CurTimeOffset == midiGetEverything, &WhoCares, pendingItem);
		/* Tell MIDI to let us get ALL AVAILABLE data (not just CURRENT data) */

	G.FastUpdate = false;				
	MyChangeState(G.FastUpdate, &WhoCares, fastItem);
	MyChangeState(!G.FastUpdate, &WhoCares, normalItem);
		/* We want to read all data that G.CurTimeOffset allows (not just ONE MIDIPacket at a time) */

	G.RecordingNow = false;	
	G.PlayingNow = false;
	
	/* Make the Playback and Stop buttons inactive. */
	GetDItem(TheDialog, playbackItem, &ItemType, &ItemHdl, &Box);	
	HiliteControl((ControlHandle) ItemHdl, 255);

	GetDItem(TheDialog, stopItem, &ItemType, &ItemHdl, &Box);
	HiliteControl((ControlHandle) ItemHdl,255);

	
	ShowWindow(TheDialog);
	
	
	StdLogMsg("\n\nEXIT-> StartDialog()\n\n");
	
}



void
StartUp(void)
{
	short		ItemType;
	Handle		ItemHdl;
	Handle		TheIcon;
	OSErr		TheErr;
	
	
	StdLogMsg("\n\nENTER-> StartUp()\n\n");
	
	/*** Store our A5 in space allocated in A5Tools.a ***/
	SaveAwayA5();	
	
	/*
		Check for the MIDI tool presence.
		This should really quit gracefully here
		and not call any other MIDI Manager Functions.
	*/
	if (SndDispVersion(midiToolNum) == 0) 
	{
		StdAlert("MIDI Manager isn't there!  Aborting.");
		ExitToShell();
	}
			

	/*
		Sign in to MIDI
	*/
	TheIcon = GetResource('ICN#', myIcon);
	TheErr = MIDISignIn(squidCakesID,
						refConSquidCakes, 
						TheIcon,
						(Str255) C2PStr("SquidCakes") );
	if (TheErr != noErr) {
		StdAlert("Error returned by MIDISignIn()... Aborting.");
		ExitToShell();
	}

	/*
		Add the Data-Input Port.
	*/
	StdLogMsg("ADDING SquidCakes' Time Port.\n");
	G.TimeBaseRefNum = AddTimePort(squidCakesID, timeID, midiPortTypeTime, noTimeBase, 
						 midiGetNothing,  noReadHook, refConTime, "Time Base Port",
						 midiInternalSync, zeroTime, midiFormatMSec);
	if ((G.TimeBaseRefNum == squidLibErr) && (CalamariErr != midiVConnectErr)) {
		sprintf(CStrBuf1,"ERROR: squidLibErr #%d adding TimePort... Aborting.\n", CalamariErr);
		StdAlert(CStrBuf1);
		StdLogMsg(CStrBuf1);
		ExitToShell();
	}
	else {
		MIDIStartTime(G.TimeBaseRefNum); /* Start the Time Base Clock */
	}
	
	
	/*
		Add the Data-Input Port.
	*/
	StdLogMsg("Adding the Data-Input Port.\n");
	G.InPortRefNum = AddInputPort(squidCakesID, inputID, midiPortTypeInput, 
								  G.TimeBaseRefNum, midiGetNothing, (Ptr) Reader, 
								  refConInput, "Data Input Port", squidInputBufSize);
	if ((G.InPortRefNum == squidLibErr) && (CalamariErr != midiVConnectErr)) {
		sprintf(CStrBuf1,"ERROR: squidLibErr #%d adding Input Port\n", CalamariErr);
		StdAlert(CStrBuf1);
		StdLogMsg(CStrBuf1);
		ExitToShell();
	}
	

	/*
		Add the Data Output Port.
	*/
	StdLogMsg("Adding the Data Output Port.\n");
	G.OutPortRefNum = AddDataPort(squidCakesID, outputID, midiPortTypeOutput,
						 G.TimeBaseRefNum, midiGetNothing, noReadHook, 
						 refConOutput, "Sequence Output Port");
	if ((G.OutPortRefNum == squidLibErr) && (CalamariErr != midiVConnectErr)) {
		sprintf(CStrBuf1,"ERROR: squidLibErr #%d adding Output Port\n", CalamariErr);
		StdAlert(CStrBuf1);
		StdLogMsg(CStrBuf1);
		ExitToShell();
	}
		
	/*	
		MIDIPacket Type lookup table 
		'M' => MIDI MIDIPacket.
		'C' => MIDI MIDIPacket.
		'n' => Reserved for future types.
	*/		
	StrCpy("MC234567", G.TypeLookupStr);	
	
	
	/*	
		MIDIPacket Status Puncutation lookup table: 
		'.' => new, not continued.
		'(' => Beginning of continued MIDIPacket.
		')' => End of continued MIDIPacket.
		'-' => Continuation.
	*/
	StrCpy(".()-", G.StatLookupStr);	


	GetDItem(TheDialog, displayItem, &ItemType, &ItemHdl, &(G.DisplayRect)); 
	InsetRect(&G.DisplayRect, 6, 6);			/*	DisplayRect UserItem stuff */
	G.DisplayRect.top = G.DisplayRect.top - 4;			
	G.LastSpot = 10000;					/*	A Very Big Number (dumb hack) */

	
	StdLogMsg("\n\nEXIT-> StartUp()\n\n");
	
}

void
RunDialog(void)
{

	EventRecord	AnEvent;
	short		ItemHit;
	Boolean		WhoCares;
	WindowPtr	WhichWindow;
	MIDIPacketPtr	P;
	short		Len;
	short		ItemType;
	Handle		ItemHdl;
	Rect		Box, Boundry;
	char		PackType, PackStatus;
	Uchar		TextMessage[256];
	short		i;
	OSErr		TheErr;

	/*
	StdLogMsg("\n\nENTER-> RunDialog()\n\n");
	*/

	AddLine("--- Hello! ---");
	G.Done = false;
	
	do {
	
		if (G.PlayingNow) {		/* currently playing back the sequence */
			
			/* if at end of sequence */
			if (G.PlayingIndex >= G.RecordingIndex) { 
			
				/* and ALL time has expired */
				if (MIDIGetCurTime(G.TimeBaseRefNum) >= G.FinalTime) {
					/* then restart the sequence */
					G.PlayingIndex = 0;		
					sprintf(TextMessage,"Resetting Playback index to %d", G.PlayingIndex);
					AddLine(TextMessage);
					MIDISetCurTime(G.TimeBaseRefNum, G.FirstTime);
					sprintf(TextMessage,"MIDISetCurTime(TimeBaseRefNum, %ld)", G.FirstTime);
					AddLine(TextMessage);
					sprintf(TextMessage,"After MIDISetCurTime(): time=%ld", MIDIGetCurTime(G.TimeBaseRefNum));
					AddLine(TextMessage);
				}
			}
			
			/* 
				While we havn't reached the end of our sequence,
				and our current playback time plus one second 
				is ahead timewise of the next MIDIPacket to be played:
					write the next MIDIPacket.
				(i.e., if the next MIDIPacket is within one second of 
				being current, then we should write it to MIDI Mgr.  
				The extra one second prevents any timing errors do to
				our own processing.)  The MIDI Mgr will
				write the MIDIPacket out at the appropriate timestamped time.
			*/
			while ( (G.PlayingIndex != G.RecordingIndex)
					/* TO WRITE ALL OUT AT ONCE: comment out next condition */
					&& 
					(MIDIGetCurTime(G.TimeBaseRefNum) + 1000 > 
					((MIDIPacketPtr) &(G.RecordingBuf[G.PlayingIndex]))->tStamp) 
					
				  )
			{
				/* Point P to the beginning of the next MIDIPacket */
				P = (MIDIPacketPtr) &(G.RecordingBuf[G.PlayingIndex]);
				
				/* Get the Length of the next MIDIPacket */
				Len = P->len;
				
				
				/* Print out what is being written */
				PackType = G.TypeLookupStr[ ExtractFlags(4, 0x07, P) ];
				PackStatus = G.StatLookupStr[ ExtractFlags(0, 0x03, P) ];
				sprintf(TextMessage,"Writing: %c %c TS=%ld : ", PackType, PackStatus, P->tStamp);
				AddLine(TextMessage);
				for (i = 0; i < (Len - 6); i++) {
					sprintf(TextMessage,"%X ", (Uchar) P->data[i]);
					AddData(TextMessage);
				}

				
				/* Write the next MIDIPacket to the output port to MIDI */
				/*Debugger();*/
				TheErr = MIDIWritePacket(G.OutPortRefNum, P);
				ReportOS(TheErr, "MIDIWritePacket");
				/* 
					Increment the Playback Index to the next MIDIPacket 
					making sure we move to an EVEN (word) offset via "Len & 1"
				*/
				G.PlayingIndex += Len + ((Len & 1) ? 1 : 0);
			}
			
		}
		
		
		if ( WaitNextEvent(everyEvent, &AnEvent, updatePeriod, NULL) ) {
		
			if ( (AnEvent.what == keyDown) && (AnEvent.modifiers & cmdKey) ) 
			{
				AdjustMenus();
				DoMenuCommand( MenuKey(AnEvent.message & charCodeMask) );
			}
		
			if ( IsDialogEvent(&AnEvent) ) {
			
				if ( DialogSelect(&AnEvent, &TheDialog, &ItemHit) ) {
				
					switch( ItemHit ) {
					
						case quitItem:
							G.Done = true;
							break;
							
						case allNotesOffItem:
							AllNotesOff(G.OutPortRefNum);
							break;
						
						case currentItem:	/* tell MIDI to send current data only */
							G.CurTimeOffset = midiGetCurrent;
							MyChangeState(true, &WhoCares, currentItem);
							MyChangeState(false, &WhoCares, pendingItem);
							break;
							
						case pendingItem:	/* tell MIDI to send all AVAILABLE data */
							G.CurTimeOffset = midiGetEverything;
							MyChangeState(false, &WhoCares, currentItem);
							MyChangeState(true, &WhoCares, pendingItem);
							break;
						
						
						case normalItem: /* read only one MIDIPacket (sent by MIDI Mgr) at a time */
						case fastItem:	 /* read ALL packets (sent by Calarmari) at once */
							MyChangeState(ItemHit == normalItem, &WhoCares, normalItem);
							MyChangeState(ItemHit == fastItem, &(G.FastUpdate), fastItem);
							break;
						
						
						case clearItem:	/* simply clear the data display */
							EraseRect( &(G.DisplayRect) );
							G.LastSpot = 10000;
							break;
						
						
						case recordItem:
							EraseRect( &(G.DisplayRect) );		/* clear display */
							G.LastSpot = 10000;
							MIDISetCurTime(G.TimeBaseRefNum,0L);	/* reset clock */
							G.RecordingIndex = 0;				/* reset rec index */
							G.RecordingNow = true;
							G.PlayingNow = false;
							
							/* 10???????? */
							GetDItem(TheDialog, recordItem, &ItemType, &ItemHdl, &Box);
							HiliteControl((ControlHandle) ItemHdl, 10);
							
							/* Un-highlight but keep active the Stop button */
							GetDItem(TheDialog, stopItem, &ItemType, &ItemHdl, &Box);
							HiliteControl((ControlHandle) ItemHdl,0);
							
							/* Deactivate the Playback button */
							GetDItem(TheDialog, playbackItem, &ItemType, &ItemHdl, &Box);
							HiliteControl((ControlHandle) ItemHdl,255);
							break;
							
							
						case playbackItem:
							EraseRect(&(G.DisplayRect));		/* clear display */
							G.LastSpot = 10000;				
							G.PlayingIndex = 0;				/* reset playback index */	
							G.PlayingNow = true;
							G.RecordingNow = false;
							
							/* set clock to the time stamp of the first message */
							MIDISetCurTime(G.TimeBaseRefNum, G.FirstTime);
							
							/* 10???????? */
							GetDItem(TheDialog, playbackItem, &ItemType, &ItemHdl, &Box);
							HiliteControl((ControlHandle) ItemHdl,10);
							
							/* Un-highlight but keep active the Stop button */
							GetDItem(TheDialog, stopItem, &ItemType, &ItemHdl, &Box);
							HiliteControl((ControlHandle) ItemHdl, 0);
							
							/* Deactivate the Record button */
							GetDItem(TheDialog, recordItem, &ItemType, &ItemHdl, &Box);
							HiliteControl((ControlHandle) ItemHdl, 255);
							break;
						
						
						case stopItem:
							if (G.RecordingNow) {
								G.RecordingNow = false;
								/* add a pause at the end of the sequence */
								G.FinalTime += 900;	
								sprintf(TextMessage,"FinalTime = %ld", G.FinalTime);
								AddLine(TextMessage);
							}
							else {
								G.PlayingNow = false;
							}
							
							/* Deactivate the Record button */
							GetDItem(TheDialog, recordItem, &ItemType, &ItemHdl, &Box);
							HiliteControl((ControlHandle) ItemHdl, 0);
							
							/* 
								If we've recorded something, then activate Playback button;
								otherwise, deacivate Playback button
							*/
							GetDItem(TheDialog, playbackItem, &ItemType, &ItemHdl, &Box);
							if (G.RecordingIndex > 0) {
								HiliteControl((ControlHandle) ItemHdl,0);
							}
							else {
								HiliteControl((ControlHandle) ItemHdl,255);
							}
							
							/* Deactivate the Stop button */
							GetDItem(TheDialog, stopItem, &ItemType, &ItemHdl, &Box);
							HiliteControl((ControlHandle) ItemHdl,255);
							
							break;
							
							
						default:
							SysBeep(2);
							break;
							
					}	/* End switch */
					
				} 		/* End if (DialogSelect()) */
				
			} 			
			else {	/* else if NOT a Dialog Event */
			
				switch (AnEvent.what) 
				{
					case mouseDown:
						switch( FindWindow(AnEvent.where, &WhichWindow) ) 
						{
							case inMenuBar:
								AdjustMenus();
								DoMenuCommand( MenuSelect(AnEvent.where) );
								break;
							case inSysWindow:
								SystemClick(&AnEvent, WhichWindow);
								break;
							case inContent:
								if ( WhichWindow != FrontWindow() ) 
								{
									SelectWindow(WhichWindow);
									AdjustMenus();
								} 
								break;
							case inGoAway:
								if ( TrackGoAway(WhichWindow, AnEvent.where) ) 
								{
									G.Done = true;
								}
								break;
							case inDrag:
								SetRect(&Boundry, 4, 24, 
										qd.screenBits.bounds.right - 4, 
										qd.screenBits.bounds.bottom - 4
								);
								DragWindow(WhichWindow, AnEvent.where, &Boundry);
								break;
							default:
								break;
						}
						break;
						
					default:
						break;
				}				// End Switch 
			}		/* End else if NOT a Dialog Event */
		} 						
		else {		/* Else if no Event then */
			MIDIPoll(G.InPortRefNum, G.CurTimeOffset);
		}
		
		
	} while(!G.Done && !G.ShutDownNow);
	
	AddLine("--- ''Till next time! ---");
	
	
	StdLogMsg("\n\nEXIT-> RunDialog()\n\n");
	
}


void
ShutDown(void)
{
	
	
	StdLogMsg("\n\nENTER-> ShutDown()\n\n");

	
	MIDIStopTime(G.TimeBaseRefNum);
	MIDISignOut(squidCakesID);
	
	
	StdLogMsg("\n\nEXIT-> ShutDown()\n\n");
	
}


void
StopDialog(void)
{
	
	
	StdLogMsg("\n\nENTER-> StopDialog()\n\n");	
	
	HideWindow( (WindowPtr) TheDialog);
	DisposDialog(TheDialog);	
	
	StdLogMsg("\n\nEXIT-> StopDialog()\n\n");
	
}

void CloseThings(void)
{	
	StdCloseLogFile();
}

/*

NOTE: Our readhook function below is called only at non-interrupt time in response to calls to
	  MIDIPoll.  Our A5 world is guaranteed to be intact at this point.  If accesses were
	  also allowed at interrupt level, then we would have to worry about saving and setting A5.

*/
pascal short
Reader(MIDIPacketPtr ParamPacketPtr, long RefCon)
{
		
	Uchar		TextMessage[256];
	short		Length, i;		
	long		OldA5;
	MIDIPacketPtr	ThePacketPtr, P;
	Uchar		PackType, PackStatus;
	
		
	OldA5 = OurA5();	/* save current A5 and reset A5 to our A5 world */
		/* Don't need this in this case because Reader never gets called at interupt time */
	
	ThePacketPtr = (MIDIPacketPtr) ParamPacketPtr;
	
	/* Determine and display the MIDIPacket Type, Status, and Time Stamp */
	PackType = G.TypeLookupStr[ ExtractFlags(4, 0x07, ThePacketPtr) ];
	PackStatus = G.StatLookupStr[ ExtractFlags(0, 0x03, ThePacketPtr) ];
	sprintf(TextMessage,"%c %c TS=%ld : ", PackType, PackStatus, ThePacketPtr->tStamp);
	AddLine(TextMessage);
	
	/* get length of MIDIPacket (header is first 6 bytes) */
	Length = ThePacketPtr->len;	
	
	if (Length >= 6) {		/* if not, things are really screwed up in MIDI! */
		
		for (i = 0; i < (Length - 6); i++) {
			sprintf(TextMessage,"%X ", (Uchar) ThePacketPtr->data[i]);
			AddData(TextMessage);
		}
		
		
		/* if recording and we have enough room in our buffer to record more: */
		if ( G.RecordingNow && ((G.RecordingIndex+Length) < recordingBufSize-1) ) {
		
			P = (MIDIPacketPtr) &(G.RecordingBuf[G.RecordingIndex]);
			P->flags = ThePacketPtr->flags;
			P->len = ThePacketPtr->len;
			P->tStamp = ThePacketPtr->tStamp;
			
			for (i = 0; i < (Length - 6); i++) {
				P->data[i] = ThePacketPtr->data[i];
			}
			
			if (G.RecordingIndex == 0) {		/* save the first time stamp */
				G.FirstTime = P->tStamp - 100;	/* with a little extra time buffer in front */
				if (G.FirstTime < 0) {			/* WHY??? */
					G.FirstTime = 0;
				}
			}
			
			G.FinalTime = P->tStamp;		/* save the latest time stamp */
			
			/* 
				increment RecordingIndex to next MIDIPacket space 
				(plus one if it would be an odd numbered index; 
				keeps us on word boundries)
			*/
			G.RecordingIndex += Length + ( ((G.RecordingIndex + Length) & 1) ? 1 : 0 );
			
		}
			
	}
	else {
		sprintf(TextMessage,"ERROR: *** Length = %d ***", Length);
		AddData(TextMessage);
	}
	
	if ( !G.FastUpdate ) {
		MIDISetOffsetTime(G.InPortRefNum, midiGetNothing);	
			/* force only one packet to be read at this time; */
			/* basically overides the return val of midiMorePacket */
	}

	TheirA5(OldA5);	/* Restore the A5 to its previous state */
	
	return(midiMorePacket);
}


short 
ExtractFlags(shift, mask, ThePacketPtr)
short		shift, mask;
MIDIPacketPtr	ThePacketPtr;
{
	return( ((ThePacketPtr->flags >> shift) & mask) );
}


pascal void 
UpdateDisplay(DialogPtr aDialog, short itemNo)
{
	short	itemType;
	Handle	itemHdl;
	Rect	itemRect;

	GetDItem(TheDialog, itemNo, &itemType, &itemHdl, &itemRect);
	PenPat(&qd.gray);
	FrameRect(&itemRect);
}


void
AddLine(unsigned char *text)
{
	Rect	ClearRect;


	SetPort(TheDialog);
	ClipRect( &(G.DisplayRect) );
	
	G.LastSpot += lineSize;
	if (G.LastSpot > G.DisplayRect.bottom) {
		G.LastSpot = G.DisplayRect.top + lineSize;
	}
	
	RectCopy(&(G.DisplayRect), &ClearRect);	/* Function Defined in VecaStdLib */				
	ClearRect.top = G.LastSpot - lineSize;
	ClearRect.bottom = G.LastSpot + lineSize;
	EraseRect(&ClearRect);
	
	TextSize(9);
	TextFont(applFont);
	MoveTo(G.DisplayRect.left, G.LastSpot);
	drawstring(text);

	ClipRect( &(TheDialog->portRect) );
}


void
AddData(Uchar *DataStr)
{
	Point	Where;
	short	Width;
	
	GetPen(&Where);
	
	Width = stringwidth(DataStr);
	if ((Where.h + Width)  > G.DisplayRect.right) {
		AddLine("    ");
	}
		
	ClipRect( &(G.DisplayRect) );
	drawstring(DataStr);
	ClipRect( &(TheDialog->portRect) );
}


void
ReportOS(OSErr Err, Uchar *Whom)
{
	Uchar	TextMessage[256];

	if ( Err != noErr ) {
		AddLine("         ");
		sprintf(TextMessage,"ERROR # %d from %s", Err, Whom);
		AddLine(TextMessage);
		AddLine("         ");
	}
}


void
MyChangeState(Boolean state, Boolean *variable, short itemNo)
{
	short			itemType;
	Handle			itemHdl;
	Rect			itemRect;
	ControlHandle	itemCtrl;

	sprintf(CStrBuf1, "\n\nENTER-> MyChangeState(%d, %d, %d)\n\n", state,*variable, itemNo);
	StdLogMsg(CStrBuf1);
	

	GetDItem(TheDialog, itemNo, &itemType, &itemHdl, &itemRect);
	itemCtrl = (ControlHandle) itemHdl;
	
	if (state) {
		SetCtlValue(itemCtrl, 1);
	}
	else {
		SetCtlValue(itemCtrl, 0);
	}
		
	*variable = state;
	
	
	sprintf(CStrBuf1, "\n\nEXIT<- MyChangeState(%d, %d, %d)\n\n", state,*variable, itemNo);
	StdLogMsg(CStrBuf1);

}


void
MySetIHandle(short ItemNo, Handle Hand)
{
	short	ItemType;
	Handle	ItemHdl;
	Rect	ItemRect;

	GetDItem(TheDialog, ItemNo, &ItemType, &ItemHdl, &ItemRect);
	SetDItem(TheDialog, ItemNo, ItemType, Hand, &ItemRect);
}


void
SquidAllNotesOff(short PortRefNum)
{
	int KeyNum;
	
	AddLine("Turning off all notes...");
	
	for (KeyNum=0; KeyNum<128; KeyNum++) {
		SendKeyOn(PortRefNum, 0x80000000, (Byte) KeyNum, (Byte) 0);
	}
	
}

/* 
	Adds an Input Port with given Input Buffer to TheClient, 
	and returns the port's reference number.
	NOTE: this funtion may set CalamariErr and return squidLibErr.
*/
short	
AddInputPort(OSType	TheClientID, 
			OSType	ThePortID, 
			short	ThePortType,
			short	TheTimeBase,
			long	TheOffsetTime,
			Ptr		TheReadHook,
			long	TheRefcon,
			char	TheName[],
			short	BufSize
			)
{
	MIDIPortParams	Init;
	short		ReturnRefNumVal;
	char		CStrBuf1[256], CStrBuf2[256];
	OSErr		TheErr;
	
	CalamariErr = noErr;
	
	/*** Set up the Data Input Port ***/
	Init.portID		= ThePortID;
	Init.portType 	= ThePortType;
	Init.timeBase	= TheTimeBase;
	Init.offsetTime = TheOffsetTime;
	Init.readHook 	= TheReadHook;
	Init.refCon		= TheRefcon;
	C2PStrCpy(TheName, Init.name);
	
	/* Add the Data Input Port */
	sprintf(CStrBuf1,"MIDIAddPort() - AddPortMaxInputPortBuffer(%s)",
			PortType2String(ThePortType, CStrBuf2) );
	
	TheErr = MIDIAddPort(TheClientID, BufSize, &ReturnRefNumVal, &Init);
	/* maxUWord = 65535 */

	if (TheErr != noErr) {
		CalamariErr = TheErr;
	}
	
	return( (TheErr ==  noErr) ? ReturnRefNumVal : squidLibErr);
}


void 
DoMenuCommand(long MenuResult)
{
	short		MenuID;
	short		MenuItem;
	short		ItemHit;
	Str255		DAName;
	short		DARefNum;
	WindowPtr	Window;
	char		TxtMsg[256];

	Window = FrontWindow();
	MenuID = HiWord(MenuResult);	// use macros for efficiency to... 
	MenuItem = LoWord(MenuResult);	// get menu item number and menu number
	switch ( MenuID ) {
		case appleMenu:
			switch ( MenuItem ) 
			{
				case aboutMenuItem:		// bring up alert for About.
					ItemHit = Alert(sqidAboutAlertID, NULL);
					break;
					
				default:			// all non-About items in this menu are DAs
					GetItem(GetMHandle(appleMenu), MenuItem, DAName);
					DARefNum = OpenDeskAcc(DAName);
					break;
			}
			break;
		case fileMenu:
			switch ( MenuItem ) 
			{
				case closeMenuItem:
					if ( IsDAWindow(Window) )
					{
						CloseDeskAcc(((WindowPeek) Window)->windowKind);
					}
					else if ( IsAppWindow(Window) ) 
					{
						G.Done = true;
					}
					break;
					
				case quitMenuItem:
					G.Done = true;
					break;
					
				default:
					break;
			}
			break;
			
		case editMenu:
			SystemEdit(MenuItem-1);
			break;
			
		default:
			break;
	}
	HiliteMenu(0);		// Unhighlight what MenuSelect (or MenuKey) hilited 
} 


void 
AdjustMenus(void)
{
	WindowPtr	Frontmost;
	MenuHandle	menu;

	Frontmost = FrontWindow();
	
	menu = GetMHandle(editMenu);
	
	if ( IsDAWindow(Frontmost) ) 
	{
		EnableItem(menu, undoMenuItem);
		EnableItem(menu, cutMenuItem);
		EnableItem(menu, copyMenuItem);
		EnableItem(menu, pasteMenuItem);
		EnableItem(menu, clearMenuItem);
	} 
	else if ( IsAppWindow(Frontmost) ) 
	{
		DisableItem(menu, undoMenuItem);
		DisableItem(menu, cutMenuItem);
		DisableItem(menu, copyMenuItem);
		DisableItem(menu, pasteMenuItem);
		DisableItem(menu, clearMenuItem);
	}

} 


Boolean 
IsAppWindow(WindowPtr window)
{
	if ( window == nil )
	{	return false;
	}
	else	/* application windows have non-negative windowKinds */
	{	return ((WindowPeek) window)->windowKind >= 0;
	}
} 

Boolean 
IsDAWindow(WindowPtr Window)
{
	if ( Window == nil )
	{	return false;
	}
	else	/* DA windows have negative windowKinds */
	{	return ((WindowPeek) Window)->windowKind < 0;
	}
} 
